searchData={"content_type":"text/markdown","items":[{"doc":"\"Mail merging\" with Ecto structs.\n\nA parser to interpolate MediaWiki-like `[[foo.bar]]` markup using data from Ecto schemas.","ref":"Mecto.html","title":"Mecto","type":"module"},{"doc":"Extracts nodes from some markup, returning a map where the end values are the count\nof how many times that node was in the markup.\n\nNote: this doesn't validate the nodes, just extracts them or errors if the markup is\ninvalid.","ref":"Mecto.html#extract_nodes/1","title":"Mecto.extract_nodes/1","type":"function"},{"doc":"iex> Mecto.extract_nodes(\"some text [[blog_post.title]] and [[blog_post.comments[0].content]]\")\n    %{blog_post: %{title: 1, comments: %{0 => %{content: 1}}}}\n\n    iex> Mecto.extract_nodes(\"some text that [[is|invalid]]\")\n    {:error, \"invalid markup\"}","ref":"Mecto.html#extract_nodes/1-examples","title":"Examples - Mecto.extract_nodes/1","type":"function"},{"doc":"Interpolates markup with the values in the given Ecto structs.\n\nNote: this does _not_ validate the markup (beyond checking it can be parsed) - it is\nexpected that any text passed to this has already been validated with `Mecto.validate/2`","ref":"Mecto.html#interpolate/2","title":"Mecto.interpolate/2","type":"function"},{"doc":"iex> Mecto.interpolate(\"Title for post [[blog_post.title]]\", %Mecto.BlogPost{title: \"some title\"})\n    {:ok, \"Title for post some title\"}\n\n    iex> Mecto.interpolate(\"Some [[[invalid markup]]\", %Mecto.BlogPost{})\n    {:error, \"invalid markup\"}\n\n    iex> Mecto.interpolate(\"Some [[blog_post.invalid_field]]\", %Mecto.BlogPost{})\n    {:ok, \"Some \"}","ref":"Mecto.html#interpolate/2-examples","title":"Examples - Mecto.interpolate/2","type":"function"},{"doc":"Builds a map from an Ecto schema, including following associations.\n\nNote: this will not recurse if a struct is repeated deeper in the tree.","ref":"Mecto.html#parse_schema/1","title":"Mecto.parse_schema/1","type":"function"},{"doc":"iex> Mecto.parse_schema(Mecto.User)\n    %{id: :id, username: :string}\n\n    iex> Mecto.parse_schema(Mecto.Foo)\n    {:error, :missing_schema}","ref":"Mecto.html#parse_schema/1-examples","title":"Examples - Mecto.parse_schema/1","type":"function"},{"doc":"Validates markup fields exist on the supplied schema.","ref":"Mecto.html#validate/2","title":"Mecto.validate/2","type":"function"},{"doc":"iex> Mecto.validate(\"Title for post #[[blog_post.id]]\", Mecto.BlogPost)\n    %{blog_post: %{id: :id}}\n\n    iex> Mecto.validate(\"[[blog_post.invalid_field]]\", Mecto.BlogPost)\n    {:error, [\"blog_post.invalid_field does not exist\"]}","ref":"Mecto.html#validate/2-examples","title":"Examples - Mecto.validate/2","type":"function"},{"doc":"The `protocol_ex` protocol used to create schemas for custom Ecto types.","ref":"Mecto.CustomSchema.html","title":"Mecto.CustomSchema","type":"module"},{"doc":"","ref":"Mecto.CustomSchema.html#type/2","title":"Mecto.CustomSchema.type/2","type":"function"},{"doc":"Implementation of `Mecto.CustomSchema` for `Ecto.Enum`, using `protocol_ex`.","ref":"Mecto.CustomSchema.Ecto.Enum.html","title":"Mecto.CustomSchema.Ecto.Enum","type":"module"},{"doc":"","ref":"Mecto.CustomSchema.Ecto.Enum.html#type/2","title":"Mecto.CustomSchema.Ecto.Enum.type/2","type":"function"},{"doc":"Parses `\"some text with [[tag.module]]\"` into `[tag: [:module]]`","ref":"Mecto.MarkupParser.html","title":"Mecto.MarkupParser","type":"module"},{"doc":"","ref":"Mecto.MarkupParser.html#extract_nodes/1","title":"Mecto.MarkupParser.extract_nodes/1","type":"function"},{"doc":"Parses the given `binary` as parse.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the parse (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Mecto.MarkupParser.html#parse/2","title":"Mecto.MarkupParser.parse/2","type":"function"},{"doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Mecto.MarkupParser.html#parse/2-options","title":"Options - Mecto.MarkupParser.parse/2","type":"function"},{"doc":"Converts an Ecto schema into a map, for use in `Mecto.SchemaValidator`.","ref":"Mecto.SchemaExtractor.html","title":"Mecto.SchemaExtractor","type":"module"},{"doc":"","ref":"Mecto.SchemaExtractor.html#convert_from/1","title":"Mecto.SchemaExtractor.convert_from/1","type":"function"},{"doc":"","ref":"Mecto.SchemaExtractor.html#t:association/0","title":"Mecto.SchemaExtractor.association/0","type":"type"},{"doc":"","ref":"Mecto.SchemaExtractor.html#t:cardinality/0","title":"Mecto.SchemaExtractor.cardinality/0","type":"type"},{"doc":"Validates fields extracted with `Mecto.MarkupParser` exist based on the schema given from\n`Mecto.SchemaExtractor`.","ref":"Mecto.SchemaValidator.html","title":"Mecto.SchemaValidator","type":"module"},{"doc":"","ref":"Mecto.SchemaValidator.html#check/2","title":"Mecto.SchemaValidator.check/2","type":"function"},{"doc":"Used by `Mecto.SchemaValidator` to track the results of schema validation.","ref":"Mecto.SchemaValidator.Result.html","title":"Mecto.SchemaValidator.Result","type":"module"},{"doc":"","ref":"Mecto.SchemaValidator.Result.html#add_error/3","title":"Mecto.SchemaValidator.Result.add_error/3","type":"function"},{"doc":"","ref":"Mecto.SchemaValidator.Result.html#merge/3","title":"Mecto.SchemaValidator.Result.merge/3","type":"function"},{"doc":"","ref":"Mecto.SchemaValidator.Result.html#t:t/0","title":"Mecto.SchemaValidator.Result.t/0","type":"type"},{"doc":"[![build status](https://gitlab.com/mucky-pup/mecto/badges/main/pipeline.svg)](https://gitlab.com/mucky-pup/mecto/-/commits/main)\n[![coverage report](https://gitlab.com/mucky-pup/mecto/badges/main/coverage.svg)](https://mucky-pup.gitlab.io/mecto/coverage/excoveralls.html)\n[![documentation coverage](https://ik.imagekit.io/muckypup/mecto/doc-coverage.svg)](https://hexdocs.pm/mecto)\n[![License: CC BY-SA 4.0](https://img.shields.io/badge/License-CC%20BY--SA%204.0-lightgrey.svg)](https://creativecommons.org/licenses/by-sa/4.0/)\n\n# Mecto\n\n\"Mail merging\" with Ecto structs.\n\nA parser to interpolate MediaWiki-like `[[foo.bar]]` markup using data from Ecto schemas.","ref":"readme.html","title":"README","type":"extras"},{"doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `mecto` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:mecto, \"~> 0.7.0\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - README","type":"extras"},{"doc":"Mecto was originally built to provide dynamic \"mail-merging\" (think HEEx, but not compiled) from Elixir structs - specifically ones that `use Ecto.Schema`.\n\nTake this struct:\n\n```elixir\ndefmodule MyApp.BlogPost do\n  use Ecto.Schema\n\n  schema \"blog_posts\" do\n    field(:title, :string)\n    field(:content, :string)\n  end\nend\n```\n\nYou could then have some text like:\n\n```elixir\ntext = \"The latest blog post is [[blog_post.title]]\"\n```\n\nIf you wanted to validate the text has correct markup, you could call:\n\n```elixir\nMecto.validate(text, MyApp.BlogPost)\n\n> %{blog_post: %{title: :string}}\n```\n\nAnd Mecto would ensure that the fields used in the text actually exist on `MyApp.BlogPost`. You can also take it a step further,\ncalling `Mecto.interpolate` to then use the values in a specific struct:\n\n```elixir\nMecto.interpolate(text, %MyApp.BlogPost{title: \"some title\"})\n\n> {:ok, \"The latest blog post is some title\"}\n```\n\nMecto also handles relationships and custom Ecto types (i.e. defined with `Ecto.Type`).\n\nBy default there is [an implementation](https://gitlab.com/mucky-pup/mecto/-/blob/main/lib/custom_schema.ex) for `Ecto.Enum`,\nbut you can use [protocol_ex](https://github.com/OvermindDL1/protocol_ex) for your own types - this is required for them as\nElixir doesn't allow protocol dispatch on atoms (module names), only data...which Mecto doesn't have when parsing markup.\n\nSee the tests for more examples.\n\nDocumentation can be found at  .","ref":"readme.html#usage","title":"Usage - README","type":"extras"},{"doc":"If you want to support my work, you can donate on Liberapay:\n\n[![Liberapay donation link](https://liberapay.com/assets/widgets/donate.svg)](https://liberapay.com/mucky-pup-creations/donate)","ref":"readme.html#support-my-work","title":"Support my work - README","type":"extras"},{"doc":"# CHANGELOG","ref":"changelog.html","title":"CHANGELOG","type":"extras"},{"doc":"- Add more specs\n- Fix typo in `Mecto.SchemaValidator` moduledoc\n- Added note to readme explaining why protocol_ex is required\n- Update dependencies","ref":"changelog.html#v0-7-2","title":"v0.7.2 - CHANGELOG","type":"extras"},{"doc":"- Add licence\n- Add links to licence and changelog in docs","ref":"changelog.html#v0-7-1","title":"v0.7.1 - CHANGELOG","type":"extras"},{"doc":"- Add support for `Ecto.Type`\n- Add support for embedded types\n- Update docs\n- Update dependencies","ref":"changelog.html#v0-7-0","title":"v0.7.0 - CHANGELOG","type":"extras"},{"doc":"- Fix conversion of module names to snake case","ref":"changelog.html#v0-6-0","title":"v0.6.0 - CHANGELOG","type":"extras"},{"doc":"- Ensure error return of `validate` is always a list","ref":"changelog.html#v0-5-0","title":"v0.5.0 - CHANGELOG","type":"extras"},{"doc":"- Add dialyzer and (some) typespecs","ref":"changelog.html#v0-4-0","title":"v0.4.0 - CHANGELOG","type":"extras"},{"doc":"- Handle :through relationships","ref":"changelog.html#v0-3-0","title":"v0.3.0 - CHANGELOG","type":"extras"},{"doc":"- Relax nimble_parsec version requirements","ref":"changelog.html#v0-2-0","title":"v0.2.0 - CHANGELOG","type":"extras"},{"doc":"- Initial release","ref":"changelog.html#v0-1-0","title":"v0.1.0 - CHANGELOG","type":"extras"}]}